﻿// UAConsoleLiveMapping: Creates an object structure for a boiler, describes its mapping into OPC Unified Architecture server 
// using attributes, and then performs the live mapping. Boiler data is then read, written and/or subscribed to using plain .NET 
// object access.

using System;
using System.Diagnostics;
using System.Threading;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.LiveMapping;
using OpcLabs.EasyOpc.UA.LiveMapping.Extensions;
using OpcLabs.EasyOpc.UA.OperationModel;
using System.ComponentModel;
using OpcLabs.EasyOpc.UA.Navigation;
using OpcLabs.BaseLib.LiveMapping;
using PlcModel.baseTypes;
using PlcModel.PLC;

namespace PlcModel
{
    public class MAIN_PLC : DisplayData
    {
        public MAIN_PLC()
        {
            Mapper.UpdateFailure += new System.EventHandler<MappingUpdateFailureEventArgs>(c_UpdateFailureHappen);
            AddToMapping();
        }



        private DT_PLC_OPC_UA _PLC;
        public DT_PLC_OPC_UA PLC
        {
            get { return this._PLC ?? (this._PLC = new DT_PLC_OPC_UA()); }
            set
            {
                if (value == this._PLC) return;
                this._PLC = value;
                this.OnPropertyChanged();
            }
        }


        static private UAClientMapper _Mapper;
        static public UAClientMapper Mapper
        {
            get
            {
                return _Mapper ??
                  (_Mapper = new UAClientMapper());
            }
            set
            {
                if (value == _Mapper) return;
                _Mapper = value;
            }
        }

        public void Subscribing(bool active)
        {
            _Mapper.Subscribe(active);
        }

        protected void c_UpdateFailureHappen(object sender, EventArgs exceptionArgs)
        {
            Console.WriteLine("UpdateFailureHappen" + exceptionArgs.ToString());
            
        }


        public void AddToMapping()
        {

            var parse = UABrowsePath.Parse("[ObjectsFolder]/PLC_OPC_UA", "http://www.siemens.com/simatic-s7-opcua");
            try
            {
                Mapper.Map(PLC, new UAMappingContext()
                {
                    EndpointDescriptor = //"opc.tcp://10.10.31.228:4840",   // the OPC server
                new UAEndpointDescriptor()
                {
                    Authority = "213.81.164.205:4048",
                    Port = 4840,
                    UserIdentity =
                  new OpcLabs.BaseLib.IdentityModel.User.UserIdentity()
                  {
                      UserNameTokenInfo =

                  new OpcLabs.BaseLib.IdentityModel.User.UserNameTokenInfo("opclabs", "opctest53")
                  }
                },

                    // The NodeDescriptor below determines where in the OPC address space we want to map our data to.
                    NodeDescriptor = new UANodeDescriptor
                    {

                        BrowsePath = parse
                    },
                    MonitoringParameters = 100,  // requested sampling interval (for subscriptions)
                });

                Subscribing(true);

                var count = Mapper.SubscribedMonitoredItemCount;
            }
            catch (Exception ex)
            {

                throw;
            }


        }



        [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
        [UAType]
        public class DT_PLC_OPC_UA : DisplayData
        {

            private DT_DataBlocksGlobal m_DataBlocksGlobal;
            [UANode(BrowsePath = ".DataBlocksGlobal")]
            public DT_DataBlocksGlobal DataBlocksGlobal
            {
                get
                {
                    if (m_DataBlocksGlobal == null)
                    {
                        m_DataBlocksGlobal = new DT_DataBlocksGlobal();
                    }
                    return m_DataBlocksGlobal;
                }

                set
                {
                    m_DataBlocksGlobal = value;
                    this.OnPropertyChanged();
                }

            }
        }



    }
}
