﻿
using OpcLabs.EasyOpc.UA.LiveMapping;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OpcLabs.EasyOpc.UA.LiveMapping.Extensions;

using OpcLabs.BaseLib.LiveMapping;
using OpcLabs.EasyOpc.UA.LiveMapping;
using OpcLabs.EasyOpc.UA;
using PlcModel.baseTypes;

namespace Mts.Opc.BaseTypes
{
    [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
    [UAType]
    public class Ua_Int32 : Ua_NuMeric
    {
        public Ua_Int32() : base(typeof(Int32))
        {
        }

    }

    [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
    [UAType]
    public class Ua_UInt32 : Ua_NuMeric
    {
        public Ua_UInt32() : base(typeof(UInt32))
        {
        }

    }

    [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
    [UAType]
    public class Ua_UInt16 : Ua_NuMeric
    {
        public Ua_UInt16() : base(typeof(UInt16))
        {
        }

    }

    [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
    [UAType]
    public class Ua_Int16 : Ua_NuMeric
    {
        public Ua_Int16() : base(typeof(Int16))
        {
        }

    }

    [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
    [UAType]
    public class Ua_NuMeric : DisplayData
    {



        public Ua_NuMeric(Type type)
        {

            this.maxValue = Int64.MaxValue;
            this.minValue = Int64.MinValue;

            if (typeof(Byte) == type || typeof(Char) == type)
            {
                this.maxValue = byte.MaxValue;
                this.minValue = byte.MinValue;
            }

            if (typeof(Int16) == type)
            {
                this.maxValue = Int16.MaxValue;
                this.minValue = Int16.MinValue;
            }

            if (typeof(UInt16) == type)
            {
                this.maxValue = UInt16.MaxValue;
                this.minValue = UInt16.MinValue;
            }

            if (typeof(Int16) == type)
            {
                this.maxValue = Int16.MaxValue;
                this.minValue = Int16.MinValue;
            }

            if (typeof(UInt16) == type)
            {
                this.maxValue = UInt16.MaxValue;
                this.minValue = UInt16.MinValue;
            }

            //***********
            if (typeof(Int32) == type)
            {
                this.maxValue = Int32.MaxValue;
                this.minValue = Int32.MinValue;
            }

            if (typeof(UInt32) == type)
            {
                this.maxValue = UInt32.MaxValue;
                this.minValue = UInt32.MinValue;
            }

            this._TypeName = type.ToString();
        }

        Int64 maxValue;
        Int64 minValue;

        private string _TypeName;
        public string TypeName
        {
            get { return this._TypeName ?? (this._TypeName = "Unknown"); }

        }


        public void SetValue(Int64 iValue)
        {

            // todo some synchronization....

            if (InRange(iValue))
            {
                _Value = iValue;
            }

        }

        private bool InRange(Int64 Value)
        {
            return (Value >= minValue && Value <= maxValue) ? true : false;
        }

        private Int64 _Value;

     //    in parent class must by se only the[UANode(BrowsePath = ".AsDWord")] for example...

     [UAData(Operations = UADataMappingOperations.ReadAndSubscribe, Kind = UADataMappingKind.Value)]
        public Int64 Value
        {
            get
            {
                return this._Value;
            }
            set
            {
                SetValue(value);
                this.OnPropertyChanged();

            }
        }
        
        private OpcLabs.EasyOpc.UA.UANodeDescriptor _NodeDescriptor;
        [MetaMember("NodeDescriptor")]
        public OpcLabs.EasyOpc.UA.UANodeDescriptor NodeDescriptor
        {
            get { return this._NodeDescriptor ?? (this._NodeDescriptor = "unknown"); }
            set
            {
                if (value == this._NodeDescriptor) return;
                               

               this._NodeDescriptor = value;
               
                this.OnPropertyChanged();
            }
        }



        //private Exception _Exception;
        //[UAData(Operations = UADataMappingOperations.ReadAndSubscribe, Kind = UADataMappingKind.Exception)]
        //public Exception Exception
        //{
        //    get { return this._Exception; }
        //    set
        //    {
        //        this._Exception = value;
        //        this.OnPropertyChanged();
        //    }
        //}

        //private DateTime _ServerTimestamp;
        //[UAData(Operations = UADataMappingOperations.ReadAndSubscribe, Kind = UADataMappingKind.ServerTimestamp)]
        //public DateTime ServerTimestamp
        //{
        //    get { return this._ServerTimestamp; }
        //    set
        //    {
        //        this._ServerTimestamp = value;
        //        this.OnPropertyChanged();
        //    }
        //}


    }



    [UANamespace("http://www.siemens.com/simatic-s7-opcua")]
    [UAType]
    public class Ua_String : DisplayData
    {
        public Ua_String( object Parent )
        {
            _Value = "";
            _parent = Parent;
        }

       private object _parent;

        public void SetValue(string iValue)
        {
              _Value = iValue;
        }


        private string _Value;

        [UAData(Operations = UADataMappingOperations.ReadAndWrite, Kind = UADataMappingKind.Value)]
        public string Value
        {
            get
            {
                return this._Value;
            }
            set
            {
                SetValue(value);
                this.OnPropertyChanged();
            }
        }



    }


}
